@@ -13,6 +13,7 @@ public class Access {
   static public final short PROTECTED   = 0x0004;
   static public final short STATIC      = 0x0008;
   static public final short FINAL       = 0x0010;
+  static public final short SUPER       = 0x0020;
   static public final short SYNCHRONIZED= 0x0020;
   static public final short VOLATILE    = 0x0040;
   static public final short TRANSIENT   = 0x0080;
@@ -21,6 +22,14 @@ public class Access {
   static public final short ABSTRACT    = 0x0400;
 
   public static String toString(int flags)
+  {
+    return toString(flags, '\0');
+  }
+
+  /** Return a string naming the access bits in flags.
+   * @param kind 'C' for a class, 'M' for a method, 'F' for a field.
+   */
+  public static String toString(int flags, char kind)
   {
     StringBuffer buf = new StringBuffer();
     if ((flags & PUBLIC) != 0)      buf.append(" public");
@@ -28,7 +37,8 @@ public class Access {
     if ((flags & PROTECTED) != 0)   buf.append(" protected");
     if ((flags & STATIC) != 0)      buf.append(" static");
     if ((flags & FINAL) != 0)       buf.append(" final");
-    if ((flags & SYNCHRONIZED) != 0)buf.append(" synchronized");
+    if ((flags & SYNCHRONIZED) != 0)
+      buf.append(kind == 'C' ? " super" : " synchronized");
     if ((flags & VOLATILE) != 0)    buf.append(" volatile");
     if ((flags & TRANSIENT) != 0)   buf.append(" transient");
     if ((flags & NATIVE) != 0)      buf.append(" native");
