@@ -4,8 +4,8 @@
 package gnu.bytecode;
 import java.io.*;
 
-public class Variable extends Location implements java.util.Enumeration {
-
+public class Variable extends Location implements java.util.Enumeration
+{
   /* Variables in a Scope are linked together in a linked list. */
   Variable next;
   public final Variable nextVar () { return next; }
@@ -17,6 +17,21 @@ public class Variable extends Location implements java.util.Enumeration {
     return next;
   }
 
+  public Variable ()
+  {
+  }
+
+  public Variable (String name)
+  {
+    setName(name);
+  }
+
+  public Variable (String name, Type type)
+  {
+    setName(name);
+    setType(type);
+  }
+
   private int flags = SIMPLE_FLAG;
   /* The SIMPLE_FLAG records the isSimple (q.v.) state. */
   private static final int SIMPLE_FLAG = 0x1;
@@ -129,4 +144,9 @@ public class Variable extends Location implements java.util.Enumeration {
     while (--size >= 0)
       code.locals.used [offset + size] = null;
   }
+
+  public String toString()
+  {
+    return "Variable["+getName()+" offset:"+offset+']';
+  }
 }
