@@ -59,7 +59,7 @@ public class CommonLisp extends Lisp2
 
   protected void defun(String name, Object value)
   {
-    Symbol.setFunctionBinding(environ, name, value);
+    Symbols.setFunctionBinding(environ, name, value);
     if (value instanceof Named)
       {
 	Named n = (Named) value;
@@ -81,18 +81,18 @@ public class CommonLisp extends Lisp2
     environ = SymbolTable.make("interaction-environment."+(++lispCounter));
     Environment.setCurrent(environ);
 
-    TRUE = environ.getBinding("t");
+    TRUE = environ.getSymbol("t");
     TRUE.set(TRUE);
     define("nil", FALSE);
 
-    BindingEnumeration e = scmEnv.enumerateAllBindings();
+    SymbolEnumeration e = scmEnv.enumerateAllSymbols();
     while (e.hasMoreElements())
       {
-	Binding b = e.nextBinding();
-	if (b.isBound())
+	Symbol b = e.nextSymbol();
+	Object val = b.get(null);
+	if (val != null)
 	  {
 	    String name = b.getName();
-	    Object val = b.get();
 	    if (val instanceof Procedure || val instanceof kawa.lang.Syntax)
 	      defun(name, val);
 	    else
@@ -127,7 +127,10 @@ public class CommonLisp extends Lisp2
     defun("defvar", new defvar(false));
     defun("defconst", new defvar(true));
     defun("defsubst", new defun(lambda));
+    defun("function", new function(lambda));
     defun("setq", new setq());
+    defun("prog1", new prog1("prog1", 1));
+    defun("prog2", prog1.prog2);
     defun("progn", new kawa.standard.begin());
     defun("if", new kawa.standard.ifp());
     defun("or", new kawa.standard.and_or(false, this));
@@ -215,11 +218,11 @@ public class CommonLisp extends Lisp2
       name = ((Named) part).getName();
     else
       name = name.intern();
-    if (part instanceof Binding)
-      environ.addBinding((Binding) part);
+    if (part instanceof Symbol)
+      environ.addSymbol((Symbol) part);
     else if (part instanceof Procedure
 	     || part instanceof kawa.lang.Syntax)
-      Symbol.setFunctionBinding(environ, name, part);
+      Symbols.setFunctionBinding(environ, name, part);
     else
       environ.define(name, part);
   }
