@@ -1,4 +1,4 @@
-(test-init "Common Lisp tests" 5)
+(test-init "Common Lisp tests" 6)
 
 (setq y 100)
 (defun foo1 (x)
@@ -14,10 +14,13 @@
 ;(test t 'eq-test-1 (eq t 't))
 
 (test t 'equal-test-1 (equal "tt" "tt"))
-(test nil 'equal-test-1 (equal "tt" "tt "))
+(test nil 'equal-test-2 (equal "tt" "tt "))
 
 (test "The octal value of 18 is 22,
    and the hex value is 12."
       'format-test-1
       (format "The octal value of ~d is ~o,
    and the hex value is ~x." 18 18 18))
+
+(defmacro list-1 (bar) `(list ,bar))
+(test '(12) 'defmacro (list-1 12))
