@@ -1,6 +1,7 @@
 package gnu.expr;
 import gnu.mapping.*;
 import gnu.lists.*;
+import gnu.kawa.reflect.ClassMemberConstraint;
 
 /**
  * Abstract class for the dummy top-level function of a module.
@@ -10,7 +11,7 @@ import gnu.lists.*;
  * faster virtual method calls instead of slower interface calls).
  */
 
-public abstract class ModuleBody extends CpsProcedure implements Runnable
+public abstract class ModuleBody extends CpsProcedure
 {
   public void apply (CallContext stack)
   {
@@ -80,6 +81,7 @@ public abstract class ModuleBody extends CpsProcedure implements Runnable
 	CallContext ctx = CallContext.getInstance();
 	ctx.values = Values.noArgs;
 	ctx.proc = this;
+	ClassMemberConstraint.defineAll(this, ctx.getEnvironment());
 	if (getMainPrintValues())
 	  {
 	    OutPort out = OutPort.outDefault();
@@ -94,6 +96,7 @@ public abstract class ModuleBody extends CpsProcedure implements Runnable
 	  }
 	// Redundant if registerShutdownHook succeeded (e.g on JDK 1.3).
 	gnu.mapping.OutPort.runCleanups();
+	kawa.repl.exitDecrement();
       }
     catch (Throwable ex)
       {
