@@ -27,9 +27,12 @@ public class ObjectExp extends ClassExp
       {
 	LambdaExp caller = outerLambda();
 	Variable closureEnv =
-	  ! Compilation.usingTailCalls ? getHeapLambda(caller).heapFrame
+	  Compilation.defaultCallConvention < Compilation.CALL_WITH_CONSUMER ? getOwningLambda().heapFrame
 	  : caller.heapFrame != null ? caller.heapFrame	: caller.closureEnv;
-	code.emitLoad(closureEnv);
+	if (closureEnv == null)
+	  code.emitPushThis();
+	else
+	  code.emitLoad(closureEnv);
       }
     code.emitInvokeSpecial(init);
 
