@@ -61,7 +61,7 @@ public class ELisp extends Lisp2
 
   protected void defun(String name, Object value)
   {
-    gnu.commonlisp.lang.Symbol.setFunctionBinding(environ, name, value);
+    gnu.commonlisp.lang.Symbols.setFunctionBinding(environ, name, value);
     if (value instanceof Named)
       {
 	Named n = (Named) value;
@@ -83,19 +83,19 @@ public class ELisp extends Lisp2
     environ = SymbolTable.make("interaction-environment."+(++elispCounter));
     Environment.setCurrent(environ);
 
-    TRUE = environ.getBinding("t");
+    TRUE = environ.getSymbol("t");
     TRUE.set(TRUE);
     define("nil", FALSE);
 
-    BindingEnumeration e
-      = Scheme.getInstance().builtin().enumerateAllBindings();
+    SymbolEnumeration e
+      = Scheme.getInstance().builtin().enumerateAllSymbols();
     while (e.hasMoreElements())
       {
-	Binding b = e.nextBinding();
-	if (b.isBound())
+	Symbol b = e.nextSymbol();
+	Object val = b.get(null);
+	if (val != null)
 	  {
 	    String name = b.getName();
-	    Object val = b.get();
 	    if (val instanceof Procedure || val instanceof kawa.lang.Syntax)
 	      defun(name, val);
 	    else
@@ -129,6 +129,7 @@ public class ELisp extends Lisp2
     defun(NumberCompare.$Gr);
     defun(NumberCompare.$Ls$Eq);
     defun(NumberCompare.$Gr$Eq);
+    defun("self-insert-command", new gnu.jemacs.buffer.SelfInsertCommand());
 
     lambda lambda = new gnu.jemacs.lang.lambda();
     lambda.setKeywords(getSymbol("&optional"),
@@ -137,6 +138,7 @@ public class ELisp extends Lisp2
     lambda.defaultDefault = nilExpr;
     defun("lambda", lambda);
     defun("defun", new gnu.commonlisp.lang.defun(lambda));
+    defun("function", new gnu.commonlisp.lang.function(lambda));
 
     defun("defgroup", new defgroup());
     defun("defcustom", new defcustom());
@@ -144,6 +146,8 @@ public class ELisp extends Lisp2
     defun("defconst", new gnu.commonlisp.lang.defvar(true));
     defun("defsubst", new gnu.commonlisp.lang.defun(lambda));
     defun("setq", new gnu.commonlisp.lang.setq());
+    defun("prog1", gnu.commonlisp.lang.prog1.prog1);
+    defun("prog2", gnu.commonlisp.lang.prog1.prog2);
     defun("progn", new kawa.standard.begin());
     defun("if", new kawa.standard.ifp());
     defun("or", new kawa.standard.and_or(false, this));
@@ -154,7 +158,7 @@ public class ELisp extends Lisp2
     defun("save-current-buffer", new gnu.jemacs.lang.SaveExcursion(true));
     defun("let", new kawa.standard.fluid_let(false, nilExpr));
     defun("let*", new kawa.standard.fluid_let(true, nilExpr));
-    defun("concat", new kawa.standard.string_append());
+    define_field ("concat", "kawa.lib.strings", "string$Mnappend");
     Procedure not = new kawa.standard.not(this);
     defun("not", not);
     defun("null", not);
@@ -227,7 +231,7 @@ public class ELisp extends Lisp2
     else if (name == "marker")
       name = "gnu.jemacs.buffer.Marker";
     else if (name == "buffer")
-      name = "gnu.jemacs.buffer.Bufffer";
+      name = "gnu.jemacs.buffer.Buffer";
     else if (name == "window")
       name = "gnu.jemacs.buffer.Window";
     return Scheme.string2Type(name);
@@ -256,11 +260,11 @@ public class ELisp extends Lisp2
       name = ((Named) part).getName();
     else
       name = name.intern();
-    if (part instanceof Binding)
-      environ.addBinding((Binding) part);
+    if (part instanceof Symbol)
+      environ.addSymbol((Symbol) part);
     else if (part instanceof Procedure
 	     || part instanceof kawa.lang.Syntax)
-      gnu.commonlisp.lang.Symbol.setFunctionBinding(environ, name, part);
+      gnu.commonlisp.lang.Symbols.setFunctionBinding(environ, name, part);
     else
       environ.define(name, part);
   }
