@@ -1,4 +1,4 @@
-// Copyright (c) 2000, 2001  Per M.A. Bothner.
+// Copyright (c) 2000, 2001, 2003  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.kawa.functions;
@@ -112,7 +112,7 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
 
   public Expression inline (ApplyExp exp, ExpWalker walker)
   {
-    Expression folded = ApplyExp.inlineIfConstant(this, exp);
+    Expression folded = exp.inlineIfConstant(this, walker);
     if (folded != exp)
       return folded;
     Expression[] args = exp.getArgs();
@@ -153,7 +153,7 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
             if (type != null)
               {
                 PrimProcedure prim
-                  = PrimProcedure.makeBuiltinBinary(opcode, type);
+                  = PrimProcedure.makeBuiltinUnary(opcode, type);
                 return new ApplyExp(prim, args);
               }
           }
@@ -279,7 +279,7 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
       }
     else if (type.isSubtype(typeDFloNum))
       {
-	PrimType dtype = Type.double_type;
+	PrimType dtype = LangPrimType.doubleType;
 	Target dtarget = new StackTarget(dtype);
 	CodeAttr code = comp.getCode();
 	args[0].compile(comp, dtarget);
@@ -298,75 +298,61 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
     
   }
 
+  /** Classify an expression according to its numeric type.
+   * kind==0:  not a number.
+   * kind==1:  a non-real number
+   * kind==2:  real number
+   * kind==3:  floating-point
+   * kind==4:  exact integer
+   */
+  public static int classify (Type type)
+  {
+    int kind = 0;
+    if (type instanceof PrimType)
+      {
+	char sig = type.getSignature().charAt(0);
+	if (sig == 'V' || sig == 'Z' || sig == 'C')
+	  return 0;
+	else if (sig == 'D' || sig == 'F')
+	  return 3;
+	else
+	  return 4;
+      }
+    else if (type.isSubtype(typeIntNum))
+      return 4;
+    else if (type.isSubtype(typeDFloNum))
+      return 3;
+    else if (type.isSubtype(typeRealNum))
+      return 2;
+    else if (type.isSubtype(typeNumeric))
+      return 1;
+    else
+      return 0;
+  }
+
   public Type getReturnType (Expression[] args)
   {
     int len = args.length;
     if (len == 0)
       return typeIntNum;
-    Type type0 = null;
+    Type type = Type.pointer_type;
     int kind0 = 0;
-    // kind==1:  other number
-    // kind==2:  real number
-    // kind==3:  floating-point
-    // kind==4:  integer
     for (int i = 0;  i < len;  i++)
       {
 	Expression arg = args[i];
-	Type type = arg.getType();
-	int kind = 0;
-	if (type instanceof PrimType)
-	  {
-	    char sig = type.getSignature().charAt(0);
-	    if (sig == 'V' || sig == 'Z' || sig == 'C')
-	      {
-		return Type.pointer_type; // error
-	      }
-	    else if (sig == 'D' || sig == 'F')
-	      {
-		kind = 3;
-		type = typeDFloNum;
-	      }
-	    else
-	      {
-		kind = 4;
-		sig = 'I';
-		type = typeIntNum;
-	      }
-	  }
-	else
-	  {
-	    if (type.isSubtype(typeIntNum))
-	      kind = 4;
-	    else if (type.isSubtype(typeDFloNum))
-	      kind = 3;
-	    else if (type.isSubtype(typeRealNum))
-	      kind = 2;
-	    else if (type.isSubtype(typeNumeric))
-	      kind = 1;
-	    else
-	      return Type.pointer_type;
-	  }
+	int kind = classify(arg.getType());
+
+	if (kind == 0)
+	  return Type.pointer_type;
 
 	if (i == 0)
-	  {
-	    type0 = type;
-	    kind0 = kind;
-	    continue;
-	  }
-
-	if (type == type0)
-	  continue;
+	  kind0 = kind;
 
 	if (kind0 == 4 && kind == 4)
 	  type = typeIntNum;
-	else if (kind0 >= 3 && kind >= 3)
-	  {
-	    type = typeDFloNum;
-	    kind0 = 3;
-	  }
 	else if (kind0 >= 2 && kind >= 2)
 	  {
-	    if (kind0 >= 3 || kind >= 3)
+	    if (kind0 == 3 || kind == 3)
 	      {
 		type = typeDFloNum;
 		kind0 = 3;
@@ -385,6 +371,6 @@ public class AddOp extends ProcedureN implements CanInline, Inlineable
 	else
 	  return Type.pointer_type;
       }
-    return type0;
+    return type;
   }
 }
