@@ -6,13 +6,38 @@ import gnu.expr.*;
 import gnu.lists.FString;
 import java.util.Vector;
 
-public class ClassMethods extends ProcedureN
+public class ClassMethods extends Procedure2
 {
-  public int numArgs() { return 2 | (2 << 12); } // For now.
-
-  public Object applyN (Object[] args)
+  public Object apply2 (Object arg0, Object arg1)
   {
-    return apply(this, args[0], args[1], null, null, 0, 0);
+    return apply(this, arg0, arg1, null, null, 0, 0);
+  }
+
+  public static MethodProc apply (String className, String methodName)
+  {
+    String mname = methodName.equals("new") ? "<init>"
+      : Compilation.mangleName(methodName);
+    ClassType methodClass = ClassType.make(className);
+    MethodProc proc
+      = ClassMethods.apply(methodClass, mname, null, null, 0, 0);
+    if (proc == null)
+      {
+	StringBuffer sbuf = new StringBuffer();
+	if (! methodClass.isExisting())
+	  {
+	    sbuf.append("no class named ");
+	    sbuf.append(className);
+	  }
+	else
+	  {
+	    sbuf.append("no method named `");
+	    sbuf.append(mname);
+	    sbuf.append("' in class ");
+	    sbuf.append(className);
+	  }
+	throw new RuntimeException(sbuf.toString());
+      }
+    return proc;
   }
 
   public static MethodProc apply(Procedure thisProc,
@@ -27,12 +52,12 @@ public class ClassMethods extends ProcedureN
     if (arg0 instanceof ClassType)
       dtype = (ClassType) arg0;
     else if (arg0 instanceof String || arg0 instanceof FString
-	     || arg0 instanceof Binding)
+	     || arg0 instanceof Symbol)
       dtype = ClassType.make(arg0.toString());
     else
       throw new WrongType(thisProc, 0, null);
     if (arg1 instanceof String || arg1 instanceof FString
-	|| arg1 instanceof Binding)
+	|| arg1 instanceof Symbol)
       mname = arg1.toString();
     else
       throw new WrongType(thisProc, 1, null);
@@ -52,12 +77,11 @@ public class ClassMethods extends ProcedureN
                                            int modifiers, int modmask,
                                            Interpreter interpreter)
   {
-    MethodFilter filter = new MethodFilter(mname, modifiers, modmask);
-    Vector methods = new Vector();
-    dtype.getMethods(filter, "<init>".equals(mname) ? 0 : 2,
-		     methods,
-		     "-"); // Should be more specific - FIXME
+    return getMethods(dtype,mname,modifiers,modmask,false,interpreter);
+  }
 
+  private static int removeRedundantMethods(Vector methods)
+  {
     // Remove over-ridden methods.
     int mlength = methods.size();
   loopi:
@@ -90,6 +114,26 @@ public class ClassMethods extends ProcedureN
 	}
       i++;
     }
+    return mlength;
+  }
+    
+  /** Return the methods of a class with the specified name and flag.
+   * @return an array containing the methods.
+   */
+  public static PrimProcedure[] getMethods(ClassType dtype, String mname,
+                                           int modifiers, int modmask,
+                                           boolean is_special,
+                                           Interpreter interpreter)
+  {
+    MethodFilter filter = new MethodFilter(mname, modifiers, modmask);
+    boolean named_class_only = is_special || "<init>".equals(mname);
+    Vector methods = new Vector();
+    dtype.getMethods(filter, named_class_only ? 0 : 2,
+		     methods,
+		     "-"); // Should be more specific - FIXME
+
+    int mlength = (named_class_only ? methods.size()
+		   : removeRedundantMethods(methods));
 
     PrimProcedure[] result = new PrimProcedure[mlength];
     int count = 0;
@@ -98,7 +142,7 @@ public class ClassMethods extends ProcedureN
       Method method = (Method) methods.elementAt(i);
       if ((method.getModifiers() & modmask) != modifiers)
 	continue;
-      PrimProcedure pproc = new PrimProcedure(method, interpreter);
+      PrimProcedure pproc = new PrimProcedure(method, is_special, interpreter);
       result[count++] = pproc;
     }
     return result;
@@ -192,14 +236,21 @@ public class ClassMethods extends ProcedureN
     return pproc;
   }
 
-  static String checkName(Expression exp)
+  static String checkName(Expression exp, boolean reversible)
   {
     if (exp instanceof QuoteExp)
       {
         Object name = ((QuoteExp) exp).getValue();
-        if (name instanceof FString || name instanceof String
-	    || name instanceof Binding)
-          return Compilation.mangleNameIfNeeded(name.toString());
+	String nam;
+        if (name instanceof FString || name instanceof String)
+	  nam = name.toString();
+	else if (name instanceof Symbol)
+	  nam = ((Symbol) name).getName();
+	else
+	  return null;
+	if (Compilation.isValidJavaName(nam))
+	  return nam;
+	return Compilation.mangleName(nam, reversible);
       }
     return null;
   }
@@ -208,14 +259,14 @@ public class ClassMethods extends ProcedureN
 class MethodFilter implements gnu.bytecode.Filter
 {
   String name;
-  String nameV;
+  int nlen;
   int modifiers;
   int modmask;
 
   public MethodFilter(String name, int modifiers, int modmask)
   {
     this.name = name;
-    this.nameV = name+"$V";
+    this.nlen = name.length();
     this.modifiers = modifiers;
     this.modmask = modmask;
   }
@@ -224,7 +275,19 @@ class MethodFilter implements gnu.bytecode.Filter
   {
     gnu.bytecode.Method method = (gnu.bytecode.Method) value;
     String mname = method.getName();
-    return ((method.getModifiers() & modmask) == modifiers)
-      && (mname.equals(name) || mname.equals(nameV));
+    if ((method.getModifiers() & modmask) == modifiers
+	&& mname.startsWith(name))
+      {
+	int mlen = mname.length();
+	char c;
+	if (mlen == nlen
+	    || (mlen == nlen + 2
+		&& mname.charAt(nlen) == '$'
+		&& ((c = mname.charAt(nlen+1)) == 'V' || c == 'X'))
+	    || (mlen == nlen + 4
+		&& mname.endsWith("$V$X")))
+	  return true;
+      }
+    return false;
   }
 }
