@@ -4,15 +4,17 @@ import gnu.expr.*;
 import gnu.bytecode.*;
 import gnu.lists.FString;
 
-public class Invoke extends ProcedureN implements CanInline, Inlineable
+public class Invoke extends ProcedureN implements CanInline
 {
-  /** 'N' - make (new);  'S' - invoke-static;  'V'  - non-static invoke. */
+  /** 'N' - make (new);  'S' - invoke-static (static or non-static);
+      's' - like 'S' but only allow static method; 'V'  - non-static invoke. */
   char kind;
 
   Interpreter interpreter;
 
   public static final Invoke invoke = new Invoke("invoke", 'V');
   public static final Invoke invokeStatic = new Invoke("invoke-static", 'S');
+  public static final Invoke invokeSpecial = new Invoke("invoke-special", 'P');
   public static final Invoke make = new Invoke("make", 'N');
 
   public Invoke(String name, char kind)
@@ -52,10 +54,14 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
   protected static Object applyN (Invoke thisProc, Object[] args)
     throws Throwable
   {
+    int kind = thisProc.kind;
+    if (kind == 'P')
+      throw new RuntimeException(thisProc.getName() 
+                                 + ": invoke-special not allowed at run time");
+    
     int nargs = args.length;
     Procedure.checkArgCount(thisProc, nargs);
     Object arg0 = args[0];
-    int kind = thisProc.kind;
     ClassType dtype;
     String mname;
     if (kind == 'V')
@@ -66,9 +72,10 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
           arg0 = Type.make((Class) arg0);
         if (arg0 instanceof ClassType)
           dtype = (ClassType) arg0;
-        else if (arg0 instanceof String || arg0 instanceof FString
-		 || arg0 instanceof Binding)
+        else if (arg0 instanceof String || arg0 instanceof FString)
           dtype = ClassType.make(arg0.toString());
+	else if (arg0 instanceof Symbol)
+          dtype = ClassType.make(((Symbol) arg0).getName());
         else
           throw new WrongType(thisProc, 0, null);
       }
@@ -86,22 +93,23 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
     else
       {
         Object arg1 = args[1];
-        if (arg1 instanceof String || arg1 instanceof FString
-	    || arg1 instanceof Binding)
+        if (arg1 instanceof String || arg1 instanceof FString)
           mname = arg1.toString();
+	else if (arg1 instanceof Symbol)
+	  mname = ((Symbol) arg1).getName();
         else
           throw new WrongType(thisProc, 1, null);
         mname = Compilation.mangleName(mname);
       }
     MethodProc proc
       = ClassMethods.apply(dtype, mname, null, null,
-                           thisProc.kind=='S' ? Access.STATIC : 0,
-                           Access.STATIC);
+                           thisProc.kind=='s' ? Access.STATIC : 0,
+                           thisProc.kind=='S' ? 0 : Access.STATIC);
     if (proc == null)
       throw new RuntimeException(thisProc.getName() + ": no method named `"
                                  + mname + "' in class " + dtype.getName());
     Object[] margs
-      = new Object[nargs-(kind == 'S' ? 2 : staticLink != null ? 0 : 1)];
+      = new Object[nargs-(kind == 'S' || kind == 's' ? 2 : staticLink != null ? 0 : 1)];
     int i = 0;
     if (kind == 'V')
       margs[i++] = args[0];
@@ -160,22 +168,29 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
   private int cachePossiblyApplicableMethodCount;
 
   protected PrimProcedure[] getMethods(ClassType ctype, String mname,
-                                       Expression[] args, int argsToSkip)
+                                       Expression[] args, int margsLength, 
+                                       int argsStartIndex, int objIndex)
   {
     if (args == cacheArgs)
       return cacheMethods;
-    int nargs = args.length;
-    Type[] atypes = new Type[nargs - argsToSkip];
-    int i = 0;
-    if (kind == 'V')
-      atypes[i++] = ctype;
-    for ( ; i < atypes.length;  i++)
-      atypes[i] = args[i+argsToSkip].getType();
-    PrimProcedure[] methods
-    = ClassMethods.getMethods(ctype, mname,
-                              kind == 'S' ? Access.STATIC : 0, Access.STATIC,
-                              interpreter);
 
+    Type[] atypes = new Type[margsLength];
+
+    int dst = 0;
+    if (objIndex >= 0)
+      atypes[dst++] = ctype;
+    for (int src = argsStartIndex; 
+         src < args.length && dst < atypes.length; 
+         src++, dst++)
+      atypes[dst] = args[src].getType();
+
+    PrimProcedure[] methods
+      = ClassMethods.getMethods(ctype, mname,
+                                kind == 's' ? Access.STATIC : 0,
+                                kind == 'S' ? 0 : Access.STATIC,
+                                kind == 'P',
+                                interpreter);
+    
     long num = ClassMethods.selectApplicable(methods, atypes);
     cacheArgs = args;
     cacheDefinitelyApplicableMethodCount = (int) (num >> 32);
@@ -207,44 +222,82 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
     return fields;
   }
 
-  public Expression inline (ApplyExp exp)
-  {
-    return kind == 'V' ? exp : inlineClassName(exp, 0, interpreter);
-  }
-
   /** Resolve class specifier to ClassType at inline time.
    * This is an optimization to avoid having a module-level binding
    * created for the class name. */
 
   public static Expression inlineClassName (ApplyExp exp, int carg,
-					    Interpreter interpreter)
+					    InlineCalls walker)
   {
+    Compilation comp = walker.getCompilation();
+    Interpreter interpreter = comp.getInterpreter();
     Expression[] args = exp.getArgs();
     if (args.length > carg)
       {
 	Type type = interpreter.getTypeFor(args[carg]);
-	ClassType ctype;
 	if (type instanceof PairClassType)
-	  ctype = ((PairClassType) type).instanceType;
-        else if (type instanceof ClassType)
-	  ctype = (ClassType) type;
-	else
+	  type = ((PairClassType) type).instanceType;
+	else if (! (type instanceof Type))
 	  return exp;
+	if (type instanceof ClassType && ((ClassType) type).isExisting())
+	  {
+	    try
+	      {
+		type.getReflectClass();
+	      }
+	    catch (Exception ex)
+	      {
+		comp.error('e', "unknown class: " + type.getName());
+	      }
+	  }
 	Expression[] nargs = new Expression[args.length];
 	System.arraycopy(args, 0, nargs, 0, args.length);
-	nargs[carg] = new QuoteExp(ctype);
-	return new ApplyExp(exp.getFunction(), nargs);
+	nargs[carg] = new QuoteExp(type);
+	return new ApplyExp(exp.getFunction(), nargs).setLine(exp);
       }
     return exp;
   }
 
-  public void compile (ApplyExp exp, Compilation comp, Target target)
+  public Expression inline (ApplyExp exp, ExpWalker walker)
   {
     Expression[] args = exp.getArgs();
     int nargs = args.length;
     ClassType type = getClassType(args);
     String name = getMethodName(args);
-    if (type != null && name != null)
+
+    int margsLength, argsStartIndex, objIndex;
+    if (kind == 'V')                     // Invoke virtual
+      {
+        margsLength = nargs - 1;
+        argsStartIndex = 2;
+        objIndex = 0;
+      }
+    else if (kind == 'N')                // make new
+      {
+        margsLength = nargs - 1;
+        argsStartIndex = 1;
+        objIndex = -1;
+      }
+    else if (kind == 'S' || kind == 's') // Invoke static
+      {
+        margsLength = nargs - 2;
+        argsStartIndex = 2;
+        objIndex = -1;
+      }
+    else if (kind == 'P')                // Invoke special
+      {
+        margsLength = nargs - 2;
+        argsStartIndex = 3;
+        objIndex = 1;
+      }
+    else
+      return walker.noteError("unknown invoke kind: " + kind);
+                
+    if (type != null && name != null
+	// We can't generate <init> until we know whether it needs
+	// a lexical link, which we don't know until FindCapturedVars is run.
+	// Unfortunately, this means make doesn't get property inlined.  FIXME
+	&& (kind != 'N' || type.isExisting()))
       {
         PrimProcedure[] methods;
         int okCount, maybeCount;
@@ -252,11 +305,12 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
           {
             try
               {
-                methods = getMethods(type, name, args, kind == 'S' ? 2 : 1);
+                methods = getMethods(type, name, args, 
+                                     margsLength, argsStartIndex, objIndex);
               }
             catch (Exception ex)
               {
-                comp.error('w', "unknown class: " + type.getName());
+                walker.error('w', "unknown class: " + type.getName());
                 methods = null;
               }
             okCount = cacheDefinitelyApplicableMethodCount;
@@ -266,7 +320,11 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
           {
             int index = -1;
             if (methods.length == 0)
-              comp.error('w', "no method `"+name+"' in "+type.getName());
+	      {
+		if (walker.getCompilation()
+		    .getBooleanOption("warn-invoke-unknown-method", true))
+		  walker.error('w', "no method `"+name+"' in "+type.getName());
+	      }
             else if (okCount + maybeCount == 0)
               {
                 Object[] slots;
@@ -297,27 +355,24 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
                       {
                         errbuf.append(" in class ");
                         errbuf.append(type.getName());
-                        comp.error('w', errbuf.toString());
+                        walker.error('w', errbuf.toString());
                       }
                     else
                       {
                         PrimProcedure method = methods[0];
-                        CodeAttr code = comp.getCode();
-                        method.compile(new ApplyExp(method, new Expression[0]),
-                                       comp, Target.pushObject);
+			ApplyExp e = new ApplyExp(method, new Expression[0]);
                         for (int i = 0;  i < slots.length;  i++)
                           {
-                            code.emitDup(type);
-                            SlotSet.compileSet(this, type,
-                                               args[2 * i + 2],
-                                               slots[i], comp);
+			    Expression[] sargs
+			      = { e, new QuoteExp(slots[i]), args[2 * i + 2] };
+			    e = new ApplyExp(SlotSet.setFieldReturnObject,
+					     sargs);
                          }
-                        target.compileFromStack(comp, type);
-                        return;
+			return e.setLine(exp);
                       }
                   }
                 else
-                  comp.error('w', "no possibly applicable method `"
+                  walker.error('w', "no possibly applicable method `"
                              +name+"' in "+type.getName());
               }
             else if (okCount == 1 || (okCount == 0 && maybeCount == 1))
@@ -327,44 +382,65 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
                 index = MethodProc.mostSpecific(methods, okCount);
                 if (index < 0)
 		  {
-		    comp.error('w',
+		    if (kind == 'S')
+		      {
+			// If we didn't find a most specific method,
+			// check if there is one that is static.  If so,
+			// prefer that - after all, we're using invoke-static.
+			for (int i = 0;  i < okCount;  i++)
+			  {
+			    if (methods[i].getStaticFlag())
+			      {
+				if (index >= 0)
+				  {
+				    index = -1;
+				    break;
+				  }
+				else
+				  index = i;
+			      }
+			  }
+		      }
+		  }
+                if (index < 0)
+		  {
+		    walker.error('w',
 			       "more than one definitely applicable method `"
 			       +name+"' in "+type.getName());
 		    for (int i = 0;  i < okCount;  i++)
-		      comp.error('w', "candidate: " + methods[i]);
+		      walker.error('w', "candidate: " + methods[i]);
 		  }
               }
 	    else if (okCount == 0)
 	      {
-		comp.error('w',
+		walker.error('w',
 			   "no definitely applicable method `"
 			   +name+"' in "+type.getName());
 	      }
             else
 	      {
-		comp.error('w',
+		walker.error('w',
 			   "more than one possibly applicable method `"
 			   +name+"' in "+type.getName());
 		for (int i = 0;  i < okCount; )
-		  comp.error('w', "candidate: " + methods[i]);
+		  walker.error('w', "candidate: " + methods[i]);
 	      }
             if (index >= 0)
               {
-                Expression[] margs
-                  = new Expression[nargs-(kind == 'S' ? 2 : 1)];
-                int i = 0;
-                if (kind == 'V')
-                  margs[i++] = args[0];
-                System.arraycopy(args, kind == 'N' ? 1 : 2,
-                                 margs, i,
-                                 nargs - (kind == 'N' ? 1 : 2));
+                Expression[] margs = new Expression[margsLength];
+                int dst = 0;
+                if (objIndex >= 0)
+                  margs[dst++] = args[objIndex];
+                for (int src = argsStartIndex; 
+                     src < args.length && dst < margs.length; 
+                     src++, dst++)
+                  margs[dst] = args[src];
                 PrimProcedure method = methods[index];
-                method.compile(new ApplyExp(method, margs), comp, target);
-                return;
+                return new ApplyExp(method, margs).setLine(exp);
               }
           }
       }
-    ApplyExp.compile(exp, comp, target);
+    return exp;
   }
 
   private ClassType getClassType(Expression[] args)
@@ -386,42 +462,12 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
   {
     if (kind == 'N')
       return "<init>";
-    if (args.length >= 2)
-      return ClassMethods.checkName(args[1]);
+    int nameIndex = (kind == 'P' ? 2 : 1);
+    if (args.length >= nameIndex + 1)
+      return ClassMethods.checkName(args[nameIndex], false);
     return null;
   }
 
-  public synchronized Type getReturnType (Expression[] args)
-  {
-    int nargs = args.length;
-    if (nargs > 0)
-      {
-        Expression arg0 = args[0];
-        Type type = (kind == 'V' ? arg0.getType()
-                     : interpreter.getTypeFor(arg0));
-        if (kind == 'N')
-          return type == null ? Type.pointer_type : type;
-        Object name = null;
-        if (nargs >= 2
-                 && args[1] instanceof QuoteExp)
-          name = ((QuoteExp) args[1]).getValue();
-        if (type instanceof ClassType
-            && (name instanceof FString || name instanceof String
-		|| name instanceof Binding))
-          {
-            PrimProcedure[] methods = getMethods((ClassType) type,
-                                                 name.toString(), args,
-                                                 kind == 'S' ? 2 : 1);
-            if (methods != null
-                && (cacheDefinitelyApplicableMethodCount == 1
-                    || (cacheDefinitelyApplicableMethodCount == 0
-                        && cachePossiblyApplicableMethodCount == 1)))
-              return methods[0].getReturnType(args);
-          }
-      }
-    return Type.pointer_type;
-  }
-
   /** Return an ApplyExp that will call a method with given arguments.
    * @param type the class containing the method we want to call.
    * @param name the name of the method we want to call
@@ -441,7 +487,8 @@ public class Invoke extends ProcedureN implements CanInline, Inlineable
   public static synchronized PrimProcedure
   getStaticMethod(ClassType type, String name, Expression[] args)
   {
-    PrimProcedure[] methods = invokeStatic.getMethods(type, name, args, 0);
+    PrimProcedure[] methods = invokeStatic.getMethods(type, name, args, 
+                                                      args.length, 0, -1);
     int okCount = invokeStatic.cacheDefinitelyApplicableMethodCount;
     int maybeCount = invokeStatic.cachePossiblyApplicableMethodCount;
     int index;
