@@ -202,6 +202,33 @@ implements CharSeq, Externalizable, Consumable
     return new FString(copy);
   }
 
+  /** Append all the characters of another <code>FString</code>. */
+  public boolean addAll(FString s)
+  {
+    int newSize = size + s.size;
+    if (data.length < newSize)
+      setBufferLength(newSize);
+    System.arraycopy(s.data, 0, data, size, s.size);
+    size = newSize;
+    return s.size > 0;
+  }
+    
+  /** Append arguments to this FString.
+   * Used to implement Scheme's string-append and string-append/shared.
+   * @param args an array of FString value
+   * @param startIndex index of first string in <code>args</code> to use
+   */
+  public void addAllStrings(Object[] args, int startIndex)
+  {
+    int total = size;
+    for (int i = startIndex; i < args.length; ++i)
+      total += ((FString) args[i]).size;
+    setBufferLength(total);
+    
+    for (int i = startIndex; i < args.length; ++i)
+      addAll((FString) args[i]);
+  }
+  
   public String toString ()
   {
     return new String (data, 0, size);
@@ -296,7 +323,7 @@ implements CharSeq, Externalizable, Consumable
     out.write(data, 0, data.length);
   }
 
-  public boolean consumeNext(int ipos, Object xpos, Consumer out)
+  public boolean consumeNext (int ipos, Consumer out)
   {
     int index = ipos >>> 1;
     if (index >= size)
@@ -305,14 +332,16 @@ implements CharSeq, Externalizable, Consumable
     return true;
   }
 
-  protected void consume(int iposStart, Object xposStart,
-			 int iposEnd, Object xposEnd, Consumer out)
+  public void consumePosRange (int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
       return;
     int i = iposStart >>> 1;
     int end = iposEnd >>> 1;
-    out.write(data, i, end - i);
+    if (end > size)
+      end = size;
+    if (end > i)
+      out.write(data, i, end - i);
   }
 
   public void writeTo(int start, int count, java.io.Writer dest)
