@@ -214,20 +214,6 @@ public class OutPort extends PrintConsumer implements Printable
     write (i);
   }
 
-  public void writeSchemeObject (Object obj, boolean readable)
-  {
-    boolean saveReadable = printReadable;
-    try
-      {
-	printReadable = readable;
-	SFormat.print (obj, this);
-      }
-    finally
-      {
-	printReadable = saveReadable;
-      }
-  }
-
   //  java.text.FieldPosition fieldPosition;
 
   /** If non-null, use this to print numbers. */
@@ -416,11 +402,16 @@ public class OutPort extends PrintConsumer implements Printable
     writeBreak(PrettyWriter.NEWLINE_LINEAR);
   }
 
+  /** Write a new-line iff the containing section cannot be printed
+   * on one line.  Either all linear-style newlines in a logical
+   * block becomes spaces (if it all fits in a line), or none
+   * of them do. */
   public void writeBreakLinear()
   {
     writeBreak(PrettyWriter.NEWLINE_LINEAR);
   }
 
+  /** Write a new-line if needed, space otherwise. */
   public void writeSpaceFill()
   {
     write(' ');
