@@ -5,7 +5,7 @@ package gnu.mapping;
  * @author  Per Bothner
  */
 
-public abstract class Procedure implements Named, Printable
+public abstract class Procedure implements Named
 {
   /** If non-null, a sequence of (key, value)-pairs. */
   private Object[] properties;
@@ -15,7 +15,15 @@ public abstract class Procedure implements Named, Printable
 
   public String getName()
   {
-    return (String) getProperty(nameKey, null);
+    Object symbol = getProperty(nameKey, null);
+    return symbol == null ? null
+      : symbol instanceof Symbol ? ((Symbol) symbol).getName()
+      : symbol.toString();
+  }
+
+  public Object getSymbol()
+  {
+    return getProperty(nameKey, null);
   }
 
   /** @deprecated */
@@ -24,6 +32,11 @@ public abstract class Procedure implements Named, Printable
     return getName();
   }
 
+  public final void setSymbol (Object name)
+  {
+    setProperty(nameKey, name);
+  }
+
   public final void setName (String name)
   {
     setProperty(nameKey, name);
@@ -108,18 +121,18 @@ public abstract class Procedure implements Named, Printable
 	    result = apply0();
 	    break;
 	  case 1:
-	    result = apply1(ctx.getArgAsObject(0));
+	    result = apply1(ctx.getNextArg());
 	    break;
 	  case 2:
-	    result = apply2(ctx.getArgAsObject(0), ctx.getArgAsObject(1));
+	    result = apply2(ctx.getNextArg(), ctx.getNextArg());
 	    break;
 	  case 3:
-	    result = apply3(ctx.getArgAsObject(0), ctx.getArgAsObject(1),
-			    ctx.getArgAsObject(2));
+	    result = apply3(ctx.getNextArg(), ctx.getNextArg(),
+			    ctx.getNextArg());
 	    break;
 	  case 4:
-	    result = apply4(ctx.getArgAsObject(0), ctx.getArgAsObject(1),
-			    ctx.getArgAsObject(2), ctx.getArgAsObject(3));
+	    result = apply4(ctx.getNextArg(), ctx.getNextArg(),
+			    ctx.getNextArg(), ctx.getNextArg());
 	    break;
 	  default:
 	    result = applyN(ctx.getArgs());
@@ -146,6 +159,14 @@ public abstract class Procedure implements Named, Printable
     return new Setter(this);
   }
 
+  public void setSetter (Procedure setter)
+  {
+    if (this instanceof HasSetter)
+      throw new RuntimeException("procedure '"+getName()+
+				 "' has builtin setter - cannot be modified");
+    setProperty(Procedure.setterKey, setter);
+  }
+
   /** If HasSetter, the Procedure is called in the LHS of an assignment. */
   public void set0(Object result) throws Throwable
   {
@@ -162,14 +183,16 @@ public abstract class Procedure implements Named, Printable
     getSetter().applyN(args);
   }
 
-  public void print(java.io.PrintWriter ps)
+  public String toString ()
   {
-    ps.print ("#<procedure ");
+    StringBuffer sbuf = new StringBuffer();
+    sbuf.append ("#<procedure ");
     String n = getName();
     if (n == null)
       n = getClass().getName();
-    ps.print (n);
-    ps.print ('>');
+    sbuf.append(n);
+    sbuf.append('>');
+    return sbuf.toString();
   }
 
   public Object getProperty(Object key, Object defaultValue)
