@@ -2,15 +2,16 @@
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.math;
+import java.io.*;
 
 /* A unit of measurement, either primitive (meter) or derived (kilogram).
  * @author	Per Bothner
  */
 
-public class Unit extends Quantity
+public abstract class Unit extends Quantity
 {
-  String name;
   Dimensions dims;
+  /** The value of this Unit is factor*dims. */
   double factor;
   MulUnit products;
 
@@ -18,13 +19,18 @@ public class Unit extends Quantity
       Same as the value of the dimensions() only. */
   Unit base;
 
+  /** A hash table of named Units. */
+  static NamedUnit[] table = new NamedUnit[100];
+
   public final Dimensions dimensions() { return dims; }
 
   public final double doubleValue() { return factor; }
 
   public int hashCode () { return dims.hashCode (); }
 
-  static Unit mul (Unit unit1, int power1, Unit unit2, int power2)
+  public String getName() { return null; }
+
+  static Unit times(Unit unit1, int power1, Unit unit2, int power2)
   {
     // First try various simplifications.
     if (unit1 == unit2)
@@ -51,61 +57,53 @@ public class Unit extends Quantity
       {
 	MulUnit munit1 = (MulUnit) unit1;
 	if (munit1.unit1 == unit2)
-	  return mul (unit2, munit1.power1 * power1 + power2,
-		      munit1.unit2, munit1.power2 * power1);
+	  return times(unit2, munit1.power1 * power1 + power2,
+		       munit1.unit2, munit1.power2 * power1);
 	if (munit1.unit2 == unit2)
-	  return mul (munit1.unit1, munit1.power1 * power1,
-		      unit2, munit1.power2 * power1 + power2);
+	  return times(munit1.unit1, munit1.power1 * power1,
+		       unit2, munit1.power2 * power1 + power2);
 	if (unit2 instanceof MulUnit)
 	  {
 	    MulUnit munit2 = (MulUnit) unit2;
 	    if (munit1.unit1 == munit2.unit1 && munit1.unit2 == munit2.unit2)
-	      return mul (munit1.unit1,
-			  munit1.power1 * power1 + munit2.power1 * power2,
-			  munit1.unit2,
-			  munit1.power2 * power1 + munit2.power2 * power2);
+	      return times(munit1.unit1,
+			   munit1.power1 * power1 + munit2.power1 * power2,
+			   munit1.unit2,
+			   munit1.power2 * power1 + munit2.power2 * power2);
 	    if (munit1.unit1 == munit2.unit2 && munit1.unit2 == munit2.unit1)
-	      return mul (munit1.unit1,
-			  munit1.power1 * power1 + munit2.power2 * power2,
-			  munit1.unit2,
-			  munit1.power2 * power1 + munit2.power1 * power2);
+	      return times(munit1.unit1,
+			   munit1.power1 * power1 + munit2.power2 * power2,
+			   munit1.unit2,
+			   munit1.power2 * power1 + munit2.power1 * power2);
 	  }
       }
     if (unit2 instanceof MulUnit)
       {
 	MulUnit munit2 = (MulUnit) unit2;
 	if (munit2.unit1 == unit1)
-	  return mul (unit1, power1 + munit2.power1 * power2,
-		      munit2.unit2, munit2.power2 * power2);
+	  return times(unit1, power1 + munit2.power1 * power2,
+		       munit2.unit2, munit2.power2 * power2);
 	if (munit2.unit2 == unit1)
-	  return mul (munit2.unit1, munit2.power1 * power2,
-		      unit1, power1 + munit2.power2 * power2);
+	  return times(munit2.unit1, munit2.power1 * power2,
+		       unit1, power1 + munit2.power2 * power2);
       }
 
-    // Search for an existing matching MulUnit.
-    for (MulUnit u = unit1.products;  u != null;  u = u.next)
-      {
-	if (u.unit1 == unit1 && u.unit2 == unit2
-	    && u.power1 == power1 && u.power2 == power2)
-	  return u;
-      }
-    // Did not find a match - create one.
-    return new MulUnit (unit1, power1, unit2, power2);
+    return MulUnit.make(unit1, power1, unit2, power2);
   }
 
-  public static Unit mul (Unit unit1, Unit unit2)
+  public static Unit times(Unit unit1, Unit unit2)
   {
-    return mul(unit1, 1, unit2, 1);
+    return times(unit1, 1, unit2, 1);
   }
 
-  public static Unit div (Unit unit1, Unit unit2)
+  public static Unit divide (Unit unit1, Unit unit2)
   {
-    return mul(unit1, 1, unit2, -1);
+    return times(unit1, 1, unit2, -1);
   }
 
   public static Unit pow (Unit unit, int power)
   {
-    return mul(unit, power, Unit.Empty, 0);
+    return times(unit, power, Unit.Empty, 0);
   }
 
   Unit ()
@@ -113,51 +111,19 @@ public class Unit extends Quantity
     factor = 1.0;
   }
 
-  public Unit (String name, double factor, Dimensions dims)
+  public static NamedUnit make (String name, Quantity value)
   {
-    this.name = name;
-    this.factor = factor;
-    this.dims = dims;
-  }
-
-  public Unit (String name, DQuantity value)
-  {
-    this (name, value.doubleValue(), value.unit().dimensions());
-  }
-
-  public Unit (String name, double factor, Unit base)
-  {
-    this (name, factor * base.doubleValue(), base.dimensions());
-  }
-
-  public static Unit define (String name, Quantity value)
-  {
-    if (value.imValue() != 0.0)
-      throw new ArithmeticException("defining "+name+" using complex value");
-    Unit unit = new Unit (name, value.reValue(), value.unit().dimensions());
-    unitTable.put (name, unit);
-    return unit;
+    return NamedUnit.make(name, value);
   }
 
   public static Unit define (String name, DQuantity value)
   {
-    Unit unit = new Unit (name, value);
-    unitTable.put (name, unit);
-    return unit;
-  }
-
-  public static Unit define (String name, double factor, Dimensions dims)
-  {
-    Unit unit = new Unit (name, factor, dims);
-    unitTable.put (name, unit);
-    return unit;
+    return new NamedUnit (name, value);
   }
 
   public static Unit define (String name, double factor, Unit base)
   {
-    Unit unit = new Unit (name, factor, base);
-    unitTable.put (name, unit);
-    return unit;
+    return new NamedUnit (name, factor, base);
   }
 
   public Complex number() { return DFloNum.one(); }
@@ -178,11 +144,12 @@ public class Unit extends Quantity
     throw new RuntimeException ("unimplemented Unit.sqrt");
   }
 
-  public static java.util.Hashtable unitTable = new java.util.Hashtable ();
+  public static BaseUnit Empty = new BaseUnit();
+  static { Dimensions.Empty.bases[0] = Empty; }
 
-  public static Unit Empty = new Unit ();
-  static {
-    Empty.dims = Dimensions.Empty;
+  public static NamedUnit lookup (String name)
+  {
+    return NamedUnit.lookup(name);
   }
 
   public String toString (double val)
@@ -213,6 +180,7 @@ public class Unit extends Quantity
 
   public String toString ()
   {
+    String name = getName();
     if (name != null)
       return name;
     else if (this == Unit.Empty)
@@ -226,11 +194,6 @@ public class Unit extends Quantity
     return this;
   }
 
-  public static Unit lookup (String name)
-  {
-    return (Unit) unitTable.get (name);
-  }
-
   public static final BaseUnit meter = new BaseUnit ("m", "Length");
   public static final BaseUnit second = new BaseUnit ("s", "Time");
   public static final BaseUnit gram = new BaseUnit ("g", "Mass");
