@@ -1,10 +1,10 @@
 package kawa.lang;
 import kawa.standard.Scheme;
-import kawa.standard.StaticFieldConstraint;
 import gnu.bytecode.Method;
 import gnu.bytecode.Variable;
 import gnu.mapping.*;
 import gnu.expr.*;
+import gnu.kawa.reflect.StaticFieldConstraint;
 import java.lang.reflect.Modifier;
 import gnu.bytecode.Type;
 import gnu.bytecode.ClassType;
@@ -12,6 +12,7 @@ import gnu.bytecode.Access;
 import gnu.text.SourceMessages;
 import gnu.lists.*;
 import gnu.kawa.lispexpr.LispInterpreter;
+import java.util.Hashtable;
 
 /** Used to translate from source to Expression.
  * The result has macros expanded, lexical names bound, etc, and is
@@ -21,14 +22,14 @@ import gnu.kawa.lispexpr.LispInterpreter;
  * each top-level form.
  */
 
-public class Translator extends Parser
+public class Translator extends Compilation
 {
-  // Current lexical scope - map name to Declaration.
-  public Environment environ;
-
   // Global environment used to look for syntax/macros.
   private Environment env;
 
+  /** Set if we're processing a define-syntax or defmacro. */
+  public Macro currentMacroDefinition;
+
   /** Return true if decl is lexical and not fluid. */
   public boolean isLexical (Declaration decl)
   {
@@ -52,23 +53,10 @@ public class Translator extends Parser
 
   private static Expression errorExp = new ErrorExp ("unknown syntax error");
 
-  public Translator (Environment env, SourceMessages messages)
+  public Translator (Interpreter interp, SourceMessages messages)
   {
-    super(messages);
-    this.env = env;
-    environ = new Environment();
-  }
-
-  public Translator (Environment env)
-  {
-    super(new SourceMessages());
-    this.env = env;
-    environ = new Environment();
-  }
-
-  public Translator ()
-  {
-    this (Environment.user());
+    super(interp, messages);
+    this.env = interp.getEnvironment();
   }
 
   public final Environment getGlobalEnvironment() { return env; }
@@ -123,37 +111,58 @@ public class Translator extends Parser
     return new ErrorExp (message);
   }
 
-  private String nameToLookup;
+  private Object nameToLookup;
 
-  Binding lookup(Object obj)
+  /** Check if declaraton is an alias for some other name.
+   * This is needed to chase identifiers renamed for hygienic macro
+   * expansion - see SyntaxRules.expand. */
+  static ReferenceExp getOriginalRef(Declaration decl)
   {
-    String name = obj.toString();
-    Binding binding = environ.lookup(name);
-    if (binding == null)
-      binding = obj instanceof Binding ? (Binding) obj : env.lookup(name);
-    else if (binding.isBound())
+    if (decl != null && decl.isAlias() && ! decl.isIndirectBinding())
       {
-	Object val1 = binding.getValue();
-	// Check for hygiene re-naming - see SyntaxRule.execute_template.
-	if (val1 instanceof String)
-	  {
-	    name = (String) val1;
-	    binding = env.lookup(name);
-	  }
+	Expression value = decl.getValue();
+	if (value instanceof ReferenceExp)
+	  return (ReferenceExp) value;
       }
-    nameToLookup = name;
-    return binding;
+    return null;
   }
 
-  Object resolve(Binding binding, boolean function)
+  /** True iff a form matches a literal symbol. */
+  public boolean matches(Object form, String literal)
   {
-    if (binding == null)
-      return null;
-    if (function && getInterpreter().hasSeparateFunctionNamespace())
-      return binding.getFunctionValue(null);
-    if (binding.isBound())
-      return binding.getValue();
-    return null;
+    if (form instanceof Symbol)
+      {
+	ReferenceExp rexp = getOriginalRef(lexical.lookup(form, -1));
+	if (rexp != null)
+	  form = rexp.getSymbol();
+      }
+    return form == literal;
+  }
+
+  public Declaration lookup(Object name, int namespace)
+  {
+    Declaration decl = lexical.lookup(name, namespace);
+    if (decl != null
+	&& (getInterpreter().getNamespaceOf(decl) & namespace) != 0)
+      return decl;
+    return lookupGlobal(name, namespace);
+  }
+
+  public Declaration lookupGlobal(Object name)
+  {
+    return lookupGlobal(name, -1);
+  }
+
+  public Declaration lookupGlobal(Object name, int namespace)
+  {
+    ModuleExp module = currentModule();
+    Declaration decl = module.lookup(name, getInterpreter(), namespace);
+    if (decl == null)
+      {
+        decl = module.getNoDefine(name);
+        decl.setIndirectBinding(true);
+      }
+    return decl;
   }
 
   /**
@@ -161,15 +170,43 @@ public class Translator extends Parser
    */
   Object getBinding (Object obj, boolean function)
   {
-    if (obj instanceof String || obj instanceof Binding)
+    if (obj instanceof String || obj instanceof Symbol)
       {
-	Binding binding = lookup(obj);
-	obj = resolve(binding, function);
+	Declaration decl = lexical.lookup(obj, function);
+	if (decl != null)
+	  {
+	    ReferenceExp rexp = getOriginalRef(decl);
+	    if (rexp != null)
+	      {
+		decl = rexp.getBinding();
+		obj = rexp.getSymbol();
+		nameToLookup = obj;
+		if (decl == null)
+		  {
+		    obj = resolve(obj, function);
+		  }
+		else
+		  {
+		    obj = decl;
+		  }
+	      }
+	    else
+	      {
+		nameToLookup = decl.getSymbol();
+		obj = decl;
+	      }
+	  }
+	else
+	  {
+	    nameToLookup = obj;
+	    obj = resolve(obj, function);
+	  }
         if (obj instanceof Syntax)
           return obj;
 	if (obj instanceof Declaration)
 	  {
-	    Expression dval = ((Declaration) obj).getValue();
+	    Expression dval
+	      = Declaration.followAliases((Declaration) obj).getValue();
 	    if (dval instanceof QuoteExp)
 	      return ((QuoteExp) dval).getValue();
 	  }
@@ -181,9 +218,10 @@ public class Translator extends Parser
 	  }
 	else
 	  {
-	    binding = env.lookup(nameToLookup);
-	    if (binding != null && binding.isBound())
-	      return binding.get();
+	    Symbol symbol = nameToLookup instanceof Symbol ? (Symbol) nameToLookup
+	      : env.lookup(nameToLookup.toString());
+	    if (symbol != null && symbol.isBound())
+	      return symbol.get();
 	  }
 	return null;
       }
@@ -217,13 +255,24 @@ public class Translator extends Parser
         Declaration decl = ref.getBinding();
 	if (decl == null)
 	  {
-            String name = ref.getName();
-            Binding binding = env.lookup(name);
-	    if (binding != null)
+	    Object sym = ref.getSymbol();
+	    Symbol symbol;
+	    String name;
+	    if (sym instanceof Symbol)
+	      {
+		symbol = (Symbol) sym;
+		name = symbol.getName();
+	      }
+	    else
+	      {
+		name = sym.toString();
+		symbol = env.lookup(name);
+	      }
+	    if (symbol != null)
 	      if (getInterpreter().hasSeparateFunctionNamespace())
-		proc = binding.getFunctionValue(null);
+		proc = symbol.getFunctionValue(null);
 	      else
-		proc = binding.getValue();
+		proc = symbol.get(null);
 	    if (proc instanceof Syntax)
 	      return apply_rewrite ((Syntax) proc, p);
             if (proc instanceof AutoloadProcedure)
@@ -238,39 +287,13 @@ public class Translator extends Parser
                   }
               }
 	  }
-        else if (decl.getFlag(Declaration.IS_SYNTAX))
-          return apply_rewrite ((Syntax) decl.getConstantValue(), p);
-
-        if (proc != null && proc instanceof Procedure
-            && ! immediate && ref.getBinding() == null)
-          {
-            Procedure pproc = (Procedure) proc;
-
-            Class procClass = PrimProcedure.getProcedureClass(pproc);
-            gnu.bytecode.Field procField;
-	    String pname = pproc.getName();
-            if (procClass != null && pname != null)
-              {
-                ClassType procType = (ClassType) Type.make(procClass);
-                procField
-		  = procType.getDeclaredField(Compilation.mangleName(pname));
-              }
-            else
-              procField = null;
-            if (procField != null)
-              {
-                int fflags = procField.getModifiers();
-                if ((fflags & Access.STATIC) != 0)
-                  {
-                    Declaration fdecl
-                      = new Declaration(pproc.getName(), procField);
-                    fdecl.noteValue(new QuoteExp(pproc));
-                    ref.setBinding(fdecl);
-                    if ((fflags & Access.FINAL) != 0)
-                      fdecl.setFlag(Declaration.IS_CONSTANT);
-                  }
-              }
-          }
+        else
+	  {
+	    decl = Declaration.followAliases(decl);
+	    proc = decl.getConstantValue();
+	    if (proc instanceof Syntax)
+	      return apply_rewrite ((Syntax) proc, p);
+	  }
 
 	ref.setProcedureName(true);
 	if (getInterpreter().hasSeparateFunctionNamespace())
@@ -279,17 +302,8 @@ public class Translator extends Parser
 
     int cdr_length = LList.listLength(cdr, false);
 
-    if (func instanceof QuoteExp)
-      {
-	proc = ((QuoteExp) func).getValue();
-	if (proc instanceof Procedure)
-	  {
-	    String msg = WrongArguments.checkArgCount((Procedure) proc,
-						      cdr_length);
-	    if (msg != null)
-	      return syntaxError(msg);
-	  }
-      }
+    if (cdr_length < 0)
+      return syntaxError("dotted list is not allowed");
 
     Expression[] args = new Expression[cdr_length];
 
@@ -320,20 +334,79 @@ public class Translator extends Parser
       return rewrite_with_position (exp, function, (PairWithPosition) exp);
     else if (exp instanceof Pair)
       return rewrite_pair ((Pair) exp);
-    else if (exp instanceof String || exp instanceof Binding)
+    else if (exp instanceof String || exp instanceof Symbol)
       {
-	Binding binding = lookup(exp);
-	Object value = resolve(binding, function);
-        boolean separate = getInterpreter().hasSeparateFunctionNamespace();
-	Declaration decl = null;
-        if (value instanceof Declaration) // ?? FIXME
+	Declaration decl = lexical.lookup(exp, function);
+	Symbol symbol = null;
+	if (decl != null)
+	  {
+	    nameToLookup = decl.getSymbol();
+	    exp = null;
+	    ReferenceExp rexp = getOriginalRef(decl);
+	    if (rexp != null)
+	      {
+		decl = rexp.getBinding();
+		if (decl == null)
+		  {
+		    exp = rexp.getSymbol();
+		    nameToLookup = exp;
+		  }
+	      }
+	  }
+	else
+	  {
+	    nameToLookup = exp;
+	  }
+	if (nameToLookup instanceof String && decl == null)
+	  {
+	    String str = (String) nameToLookup;
+	    int colon = str.indexOf(':');
+	    if (colon > 0 && colon < str.length() - 1)
+	      {
+		String prefix = str.substring(0, colon);
+		String local = str.substring(colon + 1);
+		String xprefix
+		  = (Interpreter.NAMESPACE_PREFIX+prefix).intern();
+		Object uri_decl = lexical.lookup(xprefix, function);
+		if (uri_decl instanceof Declaration)
+		  {
+		    decl = Declaration.followAliases((Declaration) uri_decl);
+		    Expression dval = decl.getValue();
+		    if (dval instanceof QuoteExp)
+		      {
+			String uri = ((QuoteExp) dval).getValue().toString();
+			return rewrite(Symbol.make(uri, local), function);
+		      }
+		  }
+		else
+		  {
+		    Object v = resolve(env.lookup(xprefix), function);
+		    if (v != null)
+		      return rewrite(Symbol.make(v.toString(), local),
+				     function);
+		    try
+		      {
+			Class cl = Class.forName(prefix);
+			return rewrite(Symbol.make("class:"+prefix, local),
+				       function);
+		      }
+		    catch (Exception ex)
+		      {
+		      }
+		  }
+	      }
+	  }
+	symbol = exp instanceof String ? env.lookup((String) exp)
+	  : (Symbol) exp;
+	Object value = resolve(symbol, function);
+	boolean separate = getInterpreter().hasSeparateFunctionNamespace();
+        if (decl != null)
           {
-            decl = (Declaration) value;
             if (! isLexical(decl)
                 || (separate && decl.isProcedureDecl()))
               decl = null;
           }
-        else if (! immediate && value instanceof Named)
+        else if (value instanceof Named)
           {
             if (value instanceof AutoloadProcedure)
               {
@@ -346,7 +419,7 @@ public class Translator extends Parser
                   }
               }
             Named proc = (Named) value;
-            Constraint constraint = binding.getConstraint();
+            Constraint constraint = symbol.getConstraint();
             if (constraint instanceof StaticFieldConstraint)
               {
                 StaticFieldConstraint fconstraint
@@ -365,11 +438,14 @@ public class Translator extends Parser
                       decl.setFlag(Declaration.IS_SYNTAX);
                   }
               }
-            else
+            else if (Compilation.inlineOk)
               {
                 decl = Declaration.getDeclaration(proc);
               }
           }
+	if (decl != null && decl.getFlag(Declaration.FIELD_OR_METHOD)
+	    && decl.isProcedureDecl() && ! function)
+	  return syntaxError("not implemented: variable reference to a method");
 	ReferenceExp rexp = new ReferenceExp (nameToLookup, decl);
 	if (separate)
 	  rexp.setFlag(ReferenceExp.PREFER_BINDING2);
@@ -403,13 +479,12 @@ public class Translator extends Parser
     PairWithPosition ppair = (PairWithPosition) pair;
     Object saved = positionPair;
     if (positionPair == null
-	|| positionPair.getFile() != current_filename
-	|| positionPair.getLine() != current_line
-	|| positionPair.getColumn() != current_column)
+	|| positionPair.getFile() != getFile()
+	|| positionPair.getLine() != getLine()
+	|| positionPair.getColumn() != getColumn())
       {
 	saved = PairWithPosition.make(Special.eof, positionPair,
-				      current_filename,
-				      current_line, current_column);
+				      getFile(), getLine(), getColumn());
       }
     setLine(pair);
     positionPair = ppair;
@@ -439,6 +514,16 @@ public class Translator extends Parser
       }
   }
 
+  /** Set the line position of the argument to the current position. */
+
+  public void setLineOf (Expression exp)
+  {
+    if (exp.getFile () == null)
+      exp.setFile(getFile());
+    if (exp.getLine () == 0)
+      exp.setLine (getLine(), getColumn());
+  }
+
   /** Extract a type from the car fo a pair. */
   public Type exp2Type(Pair typeSpecPair)
   {
@@ -477,10 +562,7 @@ public class Translator extends Parser
 	  result = rewrite_pair (pair);  // To avoid a cycle
 	else
 	  result = rewrite (exp, function);
-	if (result.getFile () == null)
-	  result.setFile(current_filename);
-	if (result.getLine () == 0)
-	  result.setLine (current_line, current_column);
+	setLineOf(result);
       }
     finally
       {
@@ -503,9 +585,9 @@ public class Translator extends Parser
 	  forms.addElement(st);
 	else
 	  {
-	    String save_filename = current_filename;
-	    int save_line = current_line;
-	    int save_column = current_column;
+	    String save_filename = getFile();
+	    int save_line = getLine();
+	    int save_column = getColumn();
 	    try
 	      {
 		setLine(st_pair);
@@ -514,9 +596,7 @@ public class Translator extends Parser
 	      }
 	    finally
 	      {
-		current_filename = save_filename;
-		current_line = save_line;
-		current_column = save_column;
+		setLine(save_filename, save_line, save_column);
 	      }
 	  }
       }
@@ -561,33 +641,40 @@ public class Translator extends Parser
 
   public Expression rewrite_body (Object exp)
   {
+    Object saved = pushPositionOf(exp);
     java.util.Vector forms = new java.util.Vector(20);
     LetExp defs = new LetExp(null);
-    if (! scan_body (exp, forms, defs))
-      return new ErrorExp("error while scanning in body");
-    return rewrite_body(forms, defs);
-  }
-
-  public Expression rewrite_body (java.util.Vector forms, LetExp defs)
-  {
-    int nforms = forms.size();
-    if (nforms == 0)
-      return syntaxError ("body with no expressions");
-    int ndecls = defs.countDecls();
-    if (ndecls != 0)
+    defs.outer = current_scope;
+    current_scope = defs;
+    try
       {
-        Expression[] inits = new Expression[ndecls];
-        for (int i = ndecls;  --i >= 0; )
-          inits[i] = QuoteExp.nullExp;
-        defs.inits = inits;
-	push(defs);
+	if (! scan_body (exp, forms, defs))
+	  return new ErrorExp("error while scanning in body");
+	int nforms = forms.size();
+	if (nforms == 0)
+	  return syntaxError ("body with no expressions");
+	int ndecls = defs.countDecls();
+	if (ndecls != 0)
+	  {
+	    Expression[] inits = new Expression[ndecls];
+	    for (int i = ndecls;  --i >= 0; )
+	      inits[i] = QuoteExp.nullExp;
+	    defs.inits = inits;
+	  }
+	Expression body = makeBody(forms, null);
+	setLineOf(body);
+	if (ndecls == 0)
+	  return body;
+	mustCompileHere();
+	defs.body = body;
+	setLineOf(defs);
+	return defs;
+      }
+    finally
+      {
+	pop(defs);
+	popPositionOf(saved);
       }
-    Expression body = makeBody(forms, null);
-    if (ndecls == 0)
-      return body;
-    defs.body = body;
-    pop(defs);
-    return defs;
   }
 
   /** Combine a list of zero or more expression forms info a "body". */
@@ -611,17 +698,14 @@ public class Translator extends Parser
       }
   }
 
-  ModuleExp module;
-
-  public final ModuleExp getModule() { return module; }
-
   public void finishModule(ModuleExp mexp, java.util.Vector forms)
   {
     boolean moduleStatic = mexp.isStatic();
     for (Declaration decl = mexp.firstDecl();
 	 decl != null;  decl = decl.nextDecl())
       {
-	if (decl.getFlag(Declaration.NOT_DEFINING))
+	if (decl.getFlag(Declaration.NOT_DEFINING)
+	    && ! decl.getFlag(Declaration.IS_UNKNOWN))
 	  {
 	    String msg1 = "'";
 	    String msg2
@@ -658,12 +742,9 @@ public class Translator extends Parser
     if (! moduleStatic)
       mexp.declareThis(null);
 
-    module = mexp;
-    int nforms = forms.size();
-    int ndecls = mexp.countDecls();
-    pushDecls(mexp);
+    setModule(mexp);
     mexp.body = makeBody(forms, mexp);
-    pop(mexp);
+    lexical.pop(mexp);
     /* DEBUGGING:
     OutPort err = OutPort.errDefault ();
     err.print ("[Re-written expression for load/compile: ");
@@ -673,88 +754,4 @@ public class Translator extends Parser
     err.flush();
     */
   }
-
-  /** Used to remember shadowed bindings in environ.
-   * For each binding, we push <old binding>, <name>.
-   * For each new scope, we push <null>. */
-  java.util.Stack shadowStack = new java.util.Stack();
-
- /**
-   * Insert decl into environ.
-   * (Used at rewrite time, not eval time.)
-   */
-  public void push (Declaration decl)
-  {
-    String sym = decl.getName();
-    if (sym == null)
-      return;
-    pushBinding(sym, decl);
-  }
-
-  /** Remove this from Translator.environ.
-   * (Used at rewrite time, not eval time.)
-   */
-  private void pop (Declaration decl)
-  {
-    String sym = decl.getName();
-    if (sym == null)
-      return;
-    popBinding();
-  }
-
-  public final void pushDecls (ScopeExp scope)
-  {
-    // shadowStack.push(null);
-    for (Declaration decl = scope.firstDecl();
-         decl != null;  decl = decl.nextDecl())
-      push(decl);
-  }
-
-  private final void popDecls (ScopeExp scope)
-  {
-    for (Declaration decl = scope.firstDecl();
-         decl != null;  decl = decl.nextDecl())
-      pop(decl);
-  }
-
-  public void push (ScopeExp scope)
-  {
-    scope.outer = current_scope;
-    if (! (scope instanceof ModuleExp))
-      mustCompileHere();
-    current_scope = scope;
-    pushDecls(scope);
-  }
-
-  public void pop (ScopeExp scope)
-  {
-    popDecls(scope);
-    current_scope = scope.outer;
-  }
-
-  /** Note a new binding, remembering old binding in the shadowStack. */
-  public void pushBinding(String name, Object value)
-  {
-    Object old = environ.put(name, value);
-    // It is possible the same Declaration may be pushed twice:  Once
-    // in scanForDefinitions and once by pushDecls.  With some care
-    // this could probably be avoided.  FIXME.
-    if (value == old)
-      return;
-    shadowStack.push(old);
-    shadowStack.push(name);
-  }
-
-  public boolean popBinding()
-  {
-    Object name = shadowStack.pop();
-    if (name == null)
-        return false;
-    Object old = shadowStack.pop();
-    if (old == null)
-      environ.remove(name);
-    else
-      environ.put(name, old);
-    return true;
-  }
 }
