@@ -1,6 +1,6 @@
 (define (exit #!optional (status :: <int> 0))
   (invoke-static <output-port> 'runCleanups)
-  (invoke-static <java.lang.System> 'exit statuc))
+  (invoke-static <java.lang.System> 'exit status))
 
 ;; FUTURE
 
