@@ -11,8 +11,8 @@ public class TracedProcedure extends ProcedureN
   public Procedure proc;
   boolean enabled;
 
-  static Binding indentation
-  = Binding.make(gnu.math.IntNum.zero(), "indentation");
+  static Symbol indentation
+  = Symbol.makeUninterned(gnu.math.IntNum.zero(), "indentation");
   static int indentationStep = 2;
 
   public TracedProcedure (Procedure proc, boolean enable)
@@ -118,7 +118,7 @@ public class TracedProcedure extends ProcedureN
   public void print(java.io.PrintWriter ps)
   {
     ps.print ("#<procedure ");
-    String n = name ();
+    String n = getName();
     if (n == null)
       ps.print ("<unnamed>");
     else
