@@ -35,23 +35,24 @@ public class define extends Syntax implements Printable
     Pair p = (Pair) st.cdr;
     Object name = p.car;
     Pair namePair = p;
-    String sym = null;
+    Object sym = null;
     boolean function = false;
-    if (name instanceof String)
+    if (name instanceof String || name instanceof Symbol)
       {
-	sym = (String) name;
+	sym = name;
       }
     else if (name instanceof Pair)
       {
         namePair = (Pair) name;
-        if (namePair.car instanceof String)
-	  sym = (String) namePair.car;
+        if (namePair.car instanceof String
+	    || namePair.car instanceof Symbol)
+	  sym = namePair.car;
 	function = true;
       }
     if (sym != null)
       {
 	Declaration decl = defs.getDefine(sym, 'w', tr);
-	tr.pushBinding(sym, decl);
+	tr.push(decl);
 	if (makePrivate)
 	  {
 	    decl.setFlag(Declaration.PRIVATE_SPECIFIED);
@@ -59,6 +60,8 @@ public class define extends Syntax implements Printable
 	  }
 	if (makeConstant)
 	  decl.setFlag(Declaration.IS_CONSTANT);
+	if (function)
+	  decl.setProcedureDecl(true);
 	Object declForm = (! function) ? (Object) decl
 	  : (Object) tr.makePair(namePair, decl, namePair.cdr);
 	p = tr.makePair(p, declForm, p.cdr);
@@ -70,7 +73,9 @@ public class define extends Syntax implements Printable
                 decl.setCanRead(true);
 		// (define (f) ...) defaults f to being read-only,
 		// unless f is assigned to in this module.
-		if (! makeConstant && name instanceof String)
+		if (! makeConstant
+		    && (name instanceof String
+			|| ! Compilation.inlineOk))
 		  decl.setCanWrite(true);
               }
           }
@@ -83,20 +88,24 @@ public class define extends Syntax implements Printable
   public Expression rewriteForm (Pair form, Translator tr)
   {
     Object obj = form.cdr;
-    String name = null;
+    Object name = null;
     Expression value = null;
     Declaration decl = null;
+    boolean not_in_body = false;
 
     if (obj instanceof Pair)
       {
 	Pair p1 = (Pair) obj;
-	if (p1.car instanceof String && p1.cdr instanceof Pair)
+	if ((p1.car instanceof String
+	     || p1.car instanceof Symbol)
+	    && p1.cdr instanceof Pair)
 	  {
 	    Pair p2 = (Pair) p1.cdr;
 	    if (p2.cdr == LList.Empty)
 	      {
-		name = (String) p1.car;
+		name = p1.car;
 		value = tr.rewrite (p2.car);
+		not_in_body = true;
 	      }
 	  }
 	else if (p1.car instanceof Declaration && p1.cdr instanceof Pair)
@@ -104,7 +113,7 @@ public class define extends Syntax implements Printable
 	    decl = (Declaration) p1.car;
 	    Pair p2 = (Pair) p1.cdr;
 	    Pair p3;
-	    if ("::" == p2.car && p2.cdr instanceof Pair
+	    if (tr.matches(p2.car, "::") && p2.cdr instanceof Pair
 		&& (p3 = (Pair) p2.cdr).cdr instanceof Pair)
 	      {
 		decl.setType(kawa.standard.prim_method.exp2Type(p3.car, tr));
@@ -113,21 +122,22 @@ public class define extends Syntax implements Printable
 	      }
 	    if (p2.cdr == LList.Empty)
 	      {
-                name = decl.getName();
+                name = decl.getSymbol();
 		value = tr.rewrite (p2.car);
 	      }
 	  }
 	else if (p1.car instanceof Pair)
 	  {
 	    Pair p2 = (Pair) p1.car;
-	    if (p2.car instanceof String)
+	    if (p2.car instanceof String || p2.car instanceof Symbol)
 	      {
-		name = (String) p2.car;
+		name = p2.car;
+		not_in_body = true;
               }
             else if (p2.car instanceof Declaration)
               {
                 decl = (Declaration) p2.car;
-                name = decl.getName();
+                name = decl.getSymbol();
               }
             if (name != null)
               {
@@ -146,6 +156,8 @@ public class define extends Syntax implements Printable
       }
     if (name == null)
       return tr.syntaxError ("invalid syntax for "+getName());
+    if (not_in_body)
+      return tr.syntaxError (getName() + " is only allowed in a <body>");
     SetExp sexp = new SetExp (name, value);
     sexp.setDefining (true);
     if (decl != null)
