@@ -3,6 +3,7 @@ import kawa.lang.*;
 import gnu.expr.*;
 import gnu.bytecode.ClassType;
 import gnu.bytecode.Method;
+import gnu.mapping.*;
 import gnu.lists.*;
 
 public class define_syntax extends Syntax
@@ -25,29 +26,41 @@ public class define_syntax extends Syntax
       {
         return tr.syntaxError(getName()+" not in a statement list");
       }
-    String name = decl.getName();
+    Object name = decl.getSymbol();
     Macro macro = (Macro) decl.getConstantValue();
     if (! (pair.cdr instanceof Pair))
       return tr.syntaxError("Missing transformation for "+form.car);
     pair = (Pair) pair.cdr;
-    Expression rule = tr.rewrite(pair.car);
-    macro.expander = rule;
+    Object expander = macro.expander;;
+    if (! (expander instanceof Procedure)
+	&& ! (expander instanceof Expression))
+      {
+	Macro savedMacro = tr.currentMacroDefinition;
+	tr.currentMacroDefinition = macro;
+	expander = tr.rewrite(macro.expander);
+	tr.currentMacroDefinition = savedMacro;
+	macro.expander = expander;
+      }
     if (! (decl.context instanceof ModuleExp))
       {
 	return QuoteExp.voidExp;
       }
     else
       {
-	if (! (rule instanceof QuoteExp)
-	    || ! (((QuoteExp) rule).getValue() instanceof java.io.Externalizable))
+	if (expander instanceof QuoteExp)
+	  expander = ((QuoteExp) expander).getValue();
+	Expression rule;
+	if (expander instanceof Procedure
+	    && expander instanceof java.io.Externalizable)
+	  rule = new QuoteExp(macro);
+	else
 	  {
 	    Expression args[] = new Expression[2];
 	    args[0] = new QuoteExp(name);
-	    args[1] = rule;
+	    args[1] = expander instanceof Expression ? (Expression) expander
+	      : new QuoteExp (expander);
 	    rule = new ApplyExp(new PrimProcedure(makeMethod), args);
 	  }
-	else
-	  rule = new QuoteExp(macro);
         SetExp result = new SetExp (decl, rule);
         result.setDefining (true);
         return result;
@@ -58,13 +71,14 @@ public class define_syntax extends Syntax
                                      ScopeExp defs, Translator tr)
   {
     Pair p;
+    Object name;
     if (! (st.cdr instanceof Pair)
-        || ! ((p = (Pair) st.cdr).car instanceof String))
+        || ! ((name = (p = (Pair) st.cdr).car) instanceof String
+	      || name instanceof Symbol))
       {
         forms.addElement(tr.syntaxError("Missing macro name for "+st.car));
         return false;
       }
-    String name = (String) p.car;
     if (! (p.cdr instanceof Pair) || (p = (Pair) p.cdr).cdr != LList.Empty)
       {
         forms.addElement(tr.syntaxError("invalid syntax for define-syntax"));
@@ -72,13 +86,13 @@ public class define_syntax extends Syntax
       }
 
     Declaration decl = defs.getDefine(name, 'w', tr);
-    Macro macro = new Macro(name);
-    macro.bind(decl);
-
+    decl.setType(typeMacro);
+    Macro macro = Macro.make(decl);
+    macro.expander = p.car;
     p = tr.makePair(st, this, new Pair(decl, p));
     forms.addElement (p);
 
-    tr.pushBinding(name, decl);
+    tr.push(decl);
     return true;
   }
 }
