@@ -15,38 +15,6 @@ public class location extends Syntax implements Printable
 {
   static private Pattern pattern = new ListPat (2, 2);
 
-  /** Assuming obj is in an lvalue context, re-write it. */
-  public static Expression rewriteArg (Object obj, Translator tr)
-  {
-    // FIXME!  The reason we need this instead of just calling tr.rewrite(obj)
-    // is because the latter will inline procedures to PrimProcedures.
-    // We should fix Translator so it doesn't do that.
-    while (obj instanceof Pair)
-      {
-	Pair pair = (Pair) obj;
-	Object proc = pair.car;
-	Object args = pair.cdr;
-	Syntax syntax = tr.check_if_Syntax (proc);
-	if (syntax == null)
-	  {
-	    int nargs = LList.length(args);
-	    Expression[] xargs = new Expression[nargs];
-	    for (int i = 0; i < nargs; i++)
-	      {
-		pair = (Pair) args;
-		xargs[i] = tr.rewrite(pair.car);
-		args = pair.cdr;
-	      }
-	    return new ApplyExp(tr.rewrite(proc), xargs);
-	  }
-	if (syntax instanceof Macro)
-	  obj = ((Macro) syntax).expand(pair, tr);
-	else
-	  return syntax.rewrite(pair.cdr, tr);
-      }
-    return tr.rewrite(obj);
-  }
-
   public Expression rewrite (Object obj, Translator tr)
   {
     if (! (obj instanceof Pair))
@@ -55,7 +23,7 @@ public class location extends Syntax implements Printable
     if (pair.cdr != LList.Empty)
       return tr.syntaxError ("extra arguments to location");
     //    Expression arg = tr.rewrite(pair.car);
-    Expression arg = rewriteArg(pair.car, tr);
+    Expression arg = tr.rewrite(pair.car);
     return location.rewrite(arg, tr);
   }
 
